@@ -9,6 +9,9 @@ from .stream_handler import StreamHandler
 
 set_plugins_loggers_config = None
 
+# removes import warning
+LogRecordSocketReceiver = None
+
 # create a outflowLogger
 outflow_logger = OutflowLogger()
 
