@@ -8,14 +8,18 @@ import os
 import pathlib
 import sys
 import traceback
+from pathlib import Path
 from hashlib import sha256
 from uuid import uuid4
 
+import cloudpickle
+
 from outflow import __version__ as outflow_version
 from outflow.core.logging import logger, set_plugins_loggers_config
 from outflow.core.pipeline import config, context, settings
 from outflow.core.pipeline.context_manager import PipelineContextManager
 from outflow.core.plugin import Plugin
+from outflow.core.tasks.io_checker import IOChecker
 from outflow.core.workflow import WorkflowManager
 
 
@@ -68,6 +72,22 @@ class Pipeline:
                 "help": "Force backend",
             },
         ),
+        "map_uuid": (
+            ["--map-uuid", "-m"],
+            {
+                "help": "Launch outflow in remote run mode with the given map uuid",
+                "type": str,
+                "default": "",
+            },
+        ),
+        "run_uuid": (
+            ["--run-uuid", "-r"],
+            {
+                "help": "Specify the run ID",
+                "type": str,
+                "default": "",
+            },
+        ),
     }
 
     base_arguments = (
@@ -83,6 +103,7 @@ class Pipeline:
     )
 
     def __init__(self, **kwargs):
+        self.backends = {}
         self.result = []
         self._kwargs = kwargs
 
@@ -115,40 +136,54 @@ class Pipeline:
             os.environ.setdefault("OUTFLOW_SETTINGS_MODULE", settings_module)
 
         # check pipeline level command line args
-        system_args, self.command_argv = self.parse_system_args(argv=argv)
+        self.system_args, self.command_argv = self.parse_system_args(argv=argv)
 
-        if system_args.settings is not None:
-            os.environ["OUTFLOW_SETTINGS_MODULE"] = system_args.settings
+        if self.system_args.map_uuid or self.system_args.run_uuid:
+            if (
+                self.system_args.map_uuid
+                and not self.system_args.run_uuid
+                or self.system_args.run_uuid
+                and not self.system_args.map_uuid
+            ):
+                raise Exception(
+                    "Arguments --map-uuid and --run-uuid must be used together"
+                )
 
-        if system_args.config is not None:
-            os.environ["OUTFLOW_CONFIG_PATH"] = (
-                pathlib.Path(system_args.config).resolve().as_posix()
-            )
+            init_remote(self.system_args.map_uuid, self.system_args.run_uuid)
+        else:
 
-        for dir_path in system_args.python_path:
-            sys.path.append(pathlib.Path(dir_path).resolve().as_posix())
+            if self.system_args.settings is not None:
+                os.environ["OUTFLOW_SETTINGS_MODULE"] = self.system_args.settings
 
-        # If true, show program's version number and exit.
-        self.display_version = system_args.version
+            if self.system_args.config is not None:
+                os.environ["OUTFLOW_CONFIG_PATH"] = (
+                    pathlib.Path(self.system_args.config).resolve().as_posix()
+                )
 
-        # setup the logger and force the verbose level if needed
-        if system_args.log_level is not None:
-            pipeline_config["logging"]["loggers"]["outflow"][
-                "level"
-            ] = system_args.log_level
+            for dir_path in self.system_args.python_path:
+                sys.path.append(pathlib.Path(dir_path).resolve().as_posix())
 
-        set_plugins_loggers_config()
+            # If true, show program's version number and exit.
+            self.display_version = self.system_args.version
 
-        logging.config.dictConfig(pipeline_config["logging"])
+            # setup the logger and force the verbose level if needed
+            if self.system_args.log_level is not None:
+                pipeline_config["logging"]["loggers"]["outflow"][
+                    "level"
+                ] = self.system_args.log_level
 
-        logger.debug(f"Config loaded: {pipeline_config}")
+            set_plugins_loggers_config()
 
-        pipeline_config["local_mode"] = system_args.local_mode
-        if system_args.backend:
-            pipeline_config["backend"] = system_args.backend
+            logging.config.dictConfig(pipeline_config["logging"])
+
+            logger.debug(f"Config loaded: {pipeline_config}")
+
+            if self.system_args.backend:
+                pipeline_config["backend"] = self.system_args.backend
+
+            self.load_backends()
 
         self.load_plugins()
-        self.load_backends()
 
     def _dispose(self):
         # close database connections, clear Model metadata and bubble up the exception if any
@@ -165,11 +200,8 @@ class Pipeline:
             model.register()
 
     def load_backends(self):
-        from outflow.core.backends.backend import Backend as DefaultBackend
         from outflow.core.pipeline import settings
 
-        self.backends = {"default": DefaultBackend}
-
         for key, value in settings.BACKENDS.items():
             try:
                 module = importlib.import_module(value)
@@ -221,12 +253,7 @@ class Pipeline:
 
         return parser.parse_known_args(argv)
 
-    def setup_backend(self, backend_name=None):
-        from outflow.core.pipeline import config
-
-        if not backend_name:
-            backend_name = config.get("backend", "default")
-
+    def setup_backend(self, backend_name):
         Backend = self.backends.get(backend_name)
         if Backend is None:
             raise ModuleNotFoundError(
@@ -237,76 +264,123 @@ class Pipeline:
 
     def run(self):
         """Run the pipeline"""
-        if self.display_version:
-            print(f"Outflow version '{outflow_version}'")
-            return 0
+        if context.map_uuid and context.run_uuid:
 
-        context.force_dry_run = self.force_dry_run
+            run_dir = Path(f"outflow_{context.run_uuid}")
 
-        command = self.root_command.parse_command_tree(self.command_argv)
+            with open(
+                run_dir / f"map_info_{context.map_uuid}.pickle", "rb"
+            ) as map_info_file:
+                map_infos = cloudpickle.load(map_info_file)
 
-        # if no argument is supplied, the help message is triggered
-        # and command tree parser returns 0
-        if command == 0:
-            return command
+            task_id = int(os.environ["SLURM_ARRAY_TASK_ID"])
 
-        context.db_untracked = command.db_untracked
-
-        backend = self.setup_backend(command.backend)
-
-        context.backend_name = backend.name
-
-        if command.db_untracked:
-            logger.debug(
-                "db_untracked=True passed to command init, this run will not be stored in the database"
+            result = map_infos["run_workflow"](
+                map_infos["inner_workflow"][task_id],
+                map_infos["generated_inputs"][task_id],
+                task_id,
+                map_infos["raise_exceptions"],
+                map_infos["external_edges"],
             )
 
-        db_config = self.create_config_in_db()
-        db_run = self.create_run_in_db(db_config)
+            filename = f"map_result_{task_id}_{context.map_uuid}"
+            with open(run_dir / filename, "wb") as result_file:
+                cloudpickle.dump(result, result_file)
 
-        return_code = 1
+        else:
+            if self.display_version:
+                print(f"Outflow version '{outflow_version}'")
+                return 0
 
-        try:
-            db_run.start()
-            with WorkflowManager():
-                task_list = command._setup_tasks()
-            if task_list:
-                backend.check_task_io(task_list)
-                self.result = backend.run(task_list=task_list)
-            db_run.success()
-            return_code = 0
-        except Exception as e:
-            db_run.fail()
-            from outflow.management.models.runtime_exception import RuntimeException
+            context.force_dry_run = self.force_dry_run
 
-            RuntimeException.create(
-                task=getattr(e, "db_task", None),
-                exception_type=e.__class__.__name__,
-                exception_msg=str(e.args),
-                traceback="".join(traceback.format_tb(e.__traceback__)),
-                time=datetime.datetime.now(),
-            )
+            command = self.root_command.parse_command_tree(self.command_argv)
 
-            # logger.print_exception() # TODO add this for file_handler but find a way to not duplicate exception in terminal
-            raise
-        finally:
-            backend.clean()
+            # if no argument is supplied, the help message is triggered
+            # and command tree parser returns 0
+            if command == 0:
+                return command
 
-        return return_code
+            context.db_untracked = command.db_untracked
+
+            backend_to_use = "default"
+            if self.system_args.backend:
+                backend_to_use = self.system_args.backend
+            elif command.backend:
+                backend_to_use = command.backend
+            elif config.get("backend", None):
+                backend_to_use = config["backend"]
+
+            backend = self.setup_backend(backend_to_use)
+
+            context.backend_name = backend.name
+
+            if command.db_untracked:
+                logger.debug(
+                    "db_untracked=True passed to command init, this run will not be stored in the database"
+                )
+
+            db_config = self.create_config_in_db()
+            db_run = self.create_run_in_db(db_config)
+
+            return_code = 1
+
+            try:
+                db_run.start()
+
+                with WorkflowManager() as top_level_workflow:
+                    workflow = top_level_workflow.workflow
+                    task_returning = command.setup_tasks()
+                    IOChecker().compute(workflow)
+                    self.result = backend.run(
+                        workflow=workflow, task_returning=task_returning
+                    )
+
+                db_run.success()
+                return_code = 0
+
+            except Exception as e:
+                db_run.fail()
+                from outflow.management.models.runtime_exception import RuntimeException
+
+                RuntimeException.create(
+                    task=getattr(e, "db_task", None),
+                    exception_type=e.__class__.__name__,
+                    exception_msg=str(e.args),
+                    traceback="".join(traceback.format_tb(e.__traceback__)),
+                    time=datetime.datetime.now(),
+                )
+
+                # logger.print_exception() # TODO add this for file_handler but find a way to not duplicate exception in terminal
+                raise
+            finally:
+                backend.clean()
+
+            return return_code
 
     @staticmethod
     def create_config_in_db():
-        from outflow.core.pipeline import config, settings
         from outflow.management.models.configuration import Configuration
 
-        cfg = config.sanitize(config.to_serializable_dict())
-        del cfg["logging"]
-        cfg_json = json.dumps(cfg)
-        settings_json = json.dumps(settings.to_serializable_dict())
-        hash = sha256(bytes(cfg_json + settings_json, encoding="utf-8")).hexdigest()
+        settings_dict = settings.to_serializable_dict()
+
+        cfg_dict = config.sanitize(config.to_serializable_dict())
+        del cfg_dict["logging"]
+        del cfg_dict["databases"]
+
+        cfg_json_str = json.dumps(cfg_dict)
+        settings_json_str = json.dumps(settings_dict)
+
+        hash = sha256(
+            bytes(cfg_json_str + settings_json_str, encoding="utf-8")
+        ).hexdigest()
+
         return Configuration.get_or_create(
             hash=hash,
-            create_method_kwargs={"config": cfg_json, "settings": settings_json},
+            create_method_kwargs={
+                "config": cfg_dict,
+                "settings": settings_dict,
+            },
         )
 
     @staticmethod
@@ -317,6 +391,7 @@ class Pipeline:
         db_run = Run.create(uuid=run_uuid, configuration=db_config, state="pending")
 
         context.db_run = db_run
+        context.run_uuid = run_uuid
         return db_run
 
     @staticmethod
@@ -350,3 +425,44 @@ def get_pipeline_states():
         "config_state": config.getstate(),
     }
     return states
+
+
+def set_pipeline_state(*, context_state, config_state, settings_state):
+    context.setstate(context_state)
+    config.setstate(config_state)
+    settings.setstate(settings_state)
+
+
+def init_remote(map_uuid, run_uuid):
+    PipelineContextManager().__enter__()
+
+    run_dir = Path(f"outflow_{run_uuid}")
+    with open(run_dir / "pipeline_states", "rb") as pipeline_states_file:
+        pipeline_states = cloudpickle.load(pipeline_states_file)
+
+    set_pipeline_state(**pipeline_states)
+
+    context.map_uuid = map_uuid
+    context.run_uuid = run_uuid
+
+    import logging.handlers
+
+    from outflow.core.logging import set_plugins_loggers_config
+
+    set_plugins_loggers_config()
+
+    logging.config.dictConfig(config["logging"])
+
+    socket_handler = logging.handlers.SocketHandler(
+        context.head_address, context.logger_port
+    )
+    socket_handler.setLevel(logging.DEBUG)
+    from outflow.core.logging.stream_handler import AddMessageSourceInfo
+
+    socket_handler.addFilter(AddMessageSourceInfo())
+
+    for logger_name in config["logging"].get("loggers", {}):
+        if logger_name == "":
+            continue
+        _logger = logging.getLogger(logger_name)
+        _logger.handlers = [socket_handler]
