@@ -2,7 +2,7 @@
 import sys
 
 import typeguard
-from outflow.core.exceptions import TypecheckerError
+from outflow.core.exceptions import IOCheckerError
 from outflow.core.logging import logger
 from outflow.core.target import Target
 from outflow.core.types import Skipped
@@ -22,17 +22,20 @@ class Task(BaseTask):
             # run automatic output detection if none were declared
             Target.output("__auto__")(self)
 
+    def _run(self, *args, **kwargs):
+        return self.run(*args, **kwargs)
+
     def __call__(self, *args, **kwargs):
         super().__call__(*args, **kwargs)
 
-        self.db_task.start()
-
         if self.skip_if_upstream_skip and any(parent.skip for parent in self.parents):
             self.skip = True
 
         if self.skip:
             return Skipped()
 
+        self.db_task.start()
+
         self.resolve_remote_inputs(kwargs)
 
         task_kwargs = {**kwargs, **self.bind_kwargs, **self.parameterized_kwargs}
@@ -41,7 +44,7 @@ class Task(BaseTask):
         self.check_argument_types(memo)
 
         try:
-            return_value = self.run(**task_kwargs)
+            return_value = self._run(**task_kwargs)
         except Exception as e:
             e.__dict__["db_task"] = self.db_task
             self.db_task.fail()
@@ -79,13 +82,13 @@ class Task(BaseTask):
         try:
             return typeguard._CallMemo(self.run, _localns, args=[], kwargs=task_kwargs)
         except TypeError as e:
-            raise TypecheckerError(f"In task '{self.name}' - {e}") from e
+            raise IOCheckerError(f"In task '{self.name}' - {e}") from e
 
     def check_argument_types(self, memo):
         try:
             typeguard.check_argument_types(memo)
         except TypeError as e:
-            raise TypecheckerError(f"In task '{self.name}' - {e}") from e
+            raise IOCheckerError(f"In task '{self.name}' - {e}") from e
 
     def check_return_type(self, return_value):
         """A custom version of typeguard.check_return_type(return_value, memo)
